import logging

import time

from typing import Dict, List, Union, Optional

from JuMonC.tasks import Plugin
from JuMonC.helpers import convertNumbers
from JuMonC import settings


logger = logging.getLogger(__name__)


class _LinuxNetworkPlugin(Plugin.Plugin):
    
    def __init__(self) -> None:
        super().__init__()
        self.__initial_interfaces = self._GetNetworkInfo()

    
    def _isWorking(self) -> bool:
        testInterfaces = self.GetNetworkInfo()
        if len(testInterfaces) == 0:
            return False
        return True

    
    def GetNetworkInfo(self) -> List[Dict[str, Union[str, Dict[str, Union[int, float, str]]]]]:
        return _getDataDiff(newData = self._GetNetworkInfo() , oldData = self.__initial_interfaces)

    
    def _GetNetworkInfo(self) -> List[Dict[str, Union[str, Dict[str, Union[int, float, str]]]]]:
        interfaces: List[Dict[str, Union[str, Dict[str, Union[int, float, str]]]]] = []
        try:
            with open("/proc/net/dev", encoding="utf8") as file:
                data = file.read()
        except FileNotFoundError:
            logging.warning("Could not find the file: \"/proc/net/dev\"", exc_info = True)
            return []
        except PermissionError: 
            logging.warning("Missing permissions for the file: \"/proc/net/dev\"", exc_info = True)
            return []
        lines = data.split("\n")[2:] # first two lines contain table headings
        for line in lines:
            if len(line.strip()) > 0:
                elements = line.split()
                interface: Dict[str, Union[str, Dict[str, Union[int, float, str]]]] = {
                    "interface": elements[0][:len(elements[0])-1],   # remove trailing :
                    "received": {
                        "Bytes": int(elements[1]),
                        "Packets": int(elements[2]),
                        "Errs": int(elements[3]),
                        "Drop": int(elements[4]),
                        "Fifo": int(elements[5]),
                        "Frame": int(elements[6]),
                        "Compressed": int(elements[7]),
                        "Multicast": int(elements[8])
                    },
                    "transmitted":   {
                        "Bytes": int(elements[9]),
                        "Packets": int(elements[10]),
                        "Errs": int(elements[11]),
                        "Drop": int(elements[12]),
                        "Fifo": int(elements[13]),
                        "Frame": int(elements[14]),
                        "Compressed": int(elements[15]),
                        "Multicast": int(elements[16])
                    }
                }
                interfaces.append(interface)
        return interfaces

    def getAvaiableInterfaces(self) -> List[str]:
        if not hasattr(self, '_interfaces'):
            self._interfaces: List[str] = []
            netInfos = self.GetNetworkInfo()
            if netInfos is None:
                return self._interfaces
            for netInfo in netInfos:
                self._interfaces.append(str(netInfo["interface"]))
        return self._interfaces

    def getAvaiableDataTypes(self) -> List[str]:
        return ["Bytes", "Packets", "Errs", "Drop", "Fifo", "Frame", "Compressed", "Multicast"]

    def getAvaiableDataTypesDescriptions(self) -> List[str]:
        return ["the bytes send and recieved",
                "the number of packets send and recieved",
                "the number of errors that occured",
                "the number of dropped packets",
                "fifo",
                "frame",
                "compressed",
                "multicast"]

    def getData(
                self, dataType: str, duration: float, interface: str, overrideHumanReadableWithValue: Optional[bool] = None
            ) -> List[Dict[str, Union[str, Dict[str, Union[int, float, str]]]]]:
        data: List[Dict[str, Union[str, Dict[str, Union[int, float, str]]]]] = self.GetNetworkInfo()
        data = _reduceInterfaces(data, interface)
        data = _reduceDataTypes(data, dataType)

        if duration >= 0.0:
            time.sleep(duration)
            newData = self.getData(dataType = dataType, duration = -1.0, interface = interface, overrideHumanReadableWithValue = False)
            oldData = data
            data = _getDataDiff(newData = newData, oldData = oldData)
            data = _divideData(data, duration, "/s")

        if overrideHumanReadableWithValue or (overrideHumanReadableWithValue is None and settings.DEFAULT_TO_HUMAN_READABLE_NUMBERS) :
            data = _makeDataHumanReadable(data)
        
        return data


plugin = _LinuxNetworkPlugin()


def _reduceInterfaces(
                    data: List[Dict[str, Union[str, Dict[str, Union[int, float, str]]]]], 
                    interface:str
                ) -> List[Dict[str, Union[str, Dict[str, Union[int, float, str]]]]]:
    if interface == "all":
        return data
    if interface in plugin.getAvaiableInterfaces():
        dataAllInterfaces = data
        data = []
        for  interfaceData in dataAllInterfaces:
            if interfaceData["interface"] == interface:
                data.append(interfaceData)
    return data


def _reduceDataTypes(
                    data: List[Dict[str, Union[str, Dict[str, Union[int, float, str]]]]], 
                    dataType: str
                ) -> List[Dict[str, Union[str, Dict[str, Union[int, float, str]]]]]:
    if dataType == "all":
        return data
    if dataType in plugin.getAvaiableDataTypes():
        dataAllDataTypes = data
        data = []
        for interfaceData in dataAllDataTypes:
            strippedData: Dict[str, Union[str, Dict[str, Union[int, float, str]]]] = {"interface": interfaceData["interface"]}
            recData = interfaceData["received"]
            if isinstance(recData, dict):
                strippedData["received"] = {dataType: recData[dataType]}
            else:
                logging.warning("Unexpected data type in LinuxNetwork._reduceDataTypes")
            transData = interfaceData["transmitted"]
            if isinstance(transData, dict):
                strippedData["transmitted"] = {dataType: transData[dataType]}
            else:
                logging.warning("Unexpected data type in LinuxNetwork._reduceDataTypes")
            data.append(strippedData)
    return data


#pylint: disable=too-many-locals
def _getDataDiff(
                newData: List[Dict[str, Union[str, Dict[str, Union[int, float, str]]]]], 
                oldData: List[Dict[str, Union[str, Dict[str, Union[int, float, str]]]]]
            ) -> List[Dict[str, Union[str, Dict[str, Union[int, float, str]]]]]:
    data: List[Dict[str, Union[str, Dict[str, Union[int, float, str]]]]] = []
    for i, interfaceDataOld in enumerate(oldData):
        interfaceDataNew = newData[i]
        difData: Dict[str, Union[str, Dict[str, Union[int, float, str]]]] = {"interface": interfaceDataNew["interface"]}
        rec: Dict[str, Union[int, float, str]] = {}
        trans: Dict[str, Union[int, float, str]] = {}
        
        recData = interfaceDataOld["received"]
        if isinstance(recData, dict):
            dictkeys = recData.keys()
        
        for dataType in dictkeys:
            old = interfaceDataOld["received"]
            new = interfaceDataNew["received"]
            if isinstance(new, dict) and isinstance(old, dict):
                newValue = new[dataType]
                oldValue = old[dataType]
                if isinstance(newValue, (int,float)) and isinstance(oldValue, (int,float)):
                    rec[dataType] = newValue - oldValue
                else:
                    logging.warning("Unexpected data type in LinuxNetwork._getDataDiff")
            else:
                logging.warning("Unexpected data type in LinuxNetwork._getDataDiff")
            
            old = interfaceDataOld["transmitted"]
            new = interfaceDataNew["transmitted"]
            if isinstance(new, dict) and isinstance(old, dict):
                newValue = new[dataType]
                oldValue = old[dataType]
                if isinstance(newValue, (int,float)) and isinstance(oldValue, (int,float)):
                    trans[dataType] = newValue - oldValue
                else:
                    logging.warning("Unexpected data type in LinuxNetwork._getDataDiff")
            else:
                logging.warning("Unexpected data type in LinuxNetwork._getDataDiff")
                
        difData["received"] = rec
        difData["transmitted"] = trans
        data.append(difData)
    return data
#pylint: enable=too-many-locals


def _divideData(
                dataIn: List[Dict[str, Union[str, Dict[str, Union[int, float, str]]]]], 
                divisor: float, 
                typeNamePostfix: str
            ) -> List[Dict[str, Union[str, Dict[str, Union[int, float, str]]]]]:
    data: List[Dict[str, Union[str, Dict[str, Union[int, float, str]]]]]= []
    for interfaceData in dataIn:
        divData: Dict[str, Union[str, Dict[str, Union[int, float, str]]]] = {"interface": interfaceData["interface"]}
        rec: Dict[str, Union[int, float, str]] = {}
        trans: Dict[str, Union[int, float, str]] = {}
        
        recData = interfaceData["received"]
        transData = interfaceData["transmitted"]
        if isinstance(recData, dict):
            dictkeys = recData.keys()
        
        for dataType in dictkeys:
            if isinstance(recData, dict):
                dataValue = recData[dataType]
                if isinstance(dataValue, (int,float)):
                    rec[dataType + typeNamePostfix] = dataValue / divisor
                else:
                    logging.warning("Unexpected data type in LinuxNetwork._divideData")
            else:
                logging.warning("Unexpected data type in LinuxNetwork._divideData")
            if isinstance(transData, dict):
                dataValue = transData[dataType]
                if isinstance(dataValue, (int,float)):
                    trans[dataType + typeNamePostfix] = dataValue / divisor
                else:
                    logging.warning("Unexpected data type in LinuxNetwork._divideData")
            else:
                logging.warning("Unexpected data type in LinuxNetwork._divideData")
                
        divData["received"] = rec
        divData["transmitted"] = trans
        data.append(divData)
    return data


def _makeDataHumanReadable(
                dataIn: List[Dict[str, Union[str, Dict[str, Union[int, float, str]]]]]
            ) -> List[Dict[str, Union[str, Dict[str, Union[int, float, str]]]]]:
    data: List[Dict[str, Union[str, Dict[str, Union[int, float, str]]]]]= []
    for interfaceData in dataIn:
        divData: Dict[str, Union[str, Dict[str, Union[int, float, str]]]] = {"interface": interfaceData["interface"]}
        rec: Dict[str, Union[int, float, str]] = {}
        trans: Dict[str, Union[int, float, str]] = {}
        
        recData = interfaceData["received"]
        transData = interfaceData["transmitted"]
        if isinstance(recData, dict):
            dictkeys = recData.keys()
        
        for dataType in dictkeys:
            if isinstance(recData, dict):
                dataValue = recData[dataType]
                if isinstance(dataValue, (int,float)):
                    (value, unitPrefix) = convertNumbers.convertBinaryPrefix(dataValue)
                    rec[unitPrefix + dataType] = value
                else:
                    logging.warning("Unexpected data type in LinuxNetwork._makeDataHumanReadable")
            else:
                logging.warning("Unexpected data type in LinuxNetwork._makeDataHumanReadable")
            if isinstance(transData, dict):
                dataValue = transData[dataType]
                if isinstance(dataValue, (int,float)):
                    (value, unitPrefix) = convertNumbers.convertBinaryPrefix(dataValue)
                    trans[unitPrefix + dataType] = value
                else:
                    logging.warning("Unexpected data type in LinuxNetwork._makeDataHumanReadable")
            else:
                logging.warning("Unexpected data type in LinuxNetwork._makeDataHumanReadable")
                
        divData["received"] = rec
        divData["transmitted"] = trans
        data.append(divData)
    return data


from werkzeug.exceptions import NotFound


from JuMonC.helpers.cmdArguments import parseCMDOptions
from JuMonC.handlers import base, main


def test_ApiPaths():
    parseCMDOptions([])
    base.start_version = 1
    base.end_version = 5
    link = main.registerRestApiPaths()
    assert( len(main.links) >= 3 + (base.end_version - base.start_version +1 ) )


from JuMonC.tasks import disk


def test_isWorking():
    plugin = disk.plugin
    assert(plugin.isWorking)

def test_readcount():
    plugin = disk.plugin
    assert(plugin.getReadCount() >= 0)

def test_writecount():
    plugin = disk.plugin
    assert(plugin.getWriteCount() >= 0)

def test_readbytes():
    plugin = disk.plugin
    assert(plugin.getReadBytes() >= 0)

def test_writebytes():
    plugin = disk.plugin
    assert(plugin.getWriteBytes() >= 0)

def test_readtime():
    plugin = disk.plugin
    assert(plugin.getReadTime() >= 0)

def test_writetime():
    plugin = disk.plugin
    assert(plugin.getWriteTime() >= 0)

def test_readMcount():
    plugin = disk.plugin
    assert(plugin.getReadMergedCount() >= 0)

def test_writemcount():
    plugin = disk.plugin
    assert(plugin.getWriteMergedCount() >= 0)

def test_busytime():
    plugin = disk.plugin
    assert(plugin.getBusyTime() >= 0)

def test_getStatusData():
    plugin = disk.plugin
    assert(len(plugin.getStatusData("write_count")) == 1)
    assert(len(plugin.getStatusData("read_count")) == 1)
    assert(len(plugin.getStatusData("write_bytes")) == 1)
    assert(len(plugin.getStatusData("read_bytes")) == 1)
    assert(len(plugin.getStatusData("write_time")) == 1)
    assert(len(plugin.getStatusData("read_time")) == 1)
    assert(len(plugin.getStatusData("write_merged_count")) == 1)
    assert(len(plugin.getStatusData("read_merged_count")) == 1)
    assert(len(plugin.getStatusData("busy_time")) == 1)
    assert(len(plugin.getStatusData("busy_time", 1)) == 1)
    assert(len(plugin.getStatusData("busy_time", 1, False)) == 1)
    assert(len(plugin.getStatusData("none_sense", 1)) == 0)

def test_getConfigData():
    plugin = disk.plugin
    assert(len(plugin.getConfigData("none_sense")) == 0)



from JuMonC.tasks import CPU


def test_isWorking():
    plugin = CPU.plugin
    assert(plugin.isWorking)

def test_load():
    plugin = CPU.plugin
    assert(plugin.getLoad(1) > 0)
    assert(plugin.getLoad(5) > 0)
    assert(plugin.getLoad(15) > 0)

def test_getStatusData():
    plugin = CPU.plugin
    assert(len(plugin.getStatusData("load", 1)) == 1)
    assert(len(plugin.getStatusData("none_sense", 1)) == 0)

def test_getConfigData():
    plugin = CPU.plugin
    assert(len(plugin.getConfigData("none_sense", 1)) == 0)


from werkzeug.exceptions import NotFound

from JuMonC.handlers import gpu
from JuMonC.helpers.cmdArguments import parseCMDOptions


def test_ApiPaths():
    version = 1
    link = gpu.registerRestApiPaths(version)
    assert(( "v" + str(version)) in link["link"])
    assert( "gpu" in link["link"])
    assert(link["isOptional"] == False)
    
def test_wrongVersionResponse():
    try:
        res = gpu.returnGPULinks(version = -1)
    except NotFound as e:
        assert( e.code == 404)


import logging

from typing import Dict, List, Union, Optional
from types import ModuleType

from JuMonC.tasks import Plugin
from JuMonC.helpers import convertNumbers
from JuMonC import settings


logger = logging.getLogger(__name__)


class _diskPlugin(Plugin.Plugin): 
    
    _psutil: Optional[ModuleType]
    
    def __init__(self) -> None:
        super().__init__()
        self.isWorking()
        if isinstance(self._psutil, ModuleType):
            self._initValues = self._psutil.disk_io_counters()
        else:
            logging.error("Something went wrong, in disk plugin psutil is not ModuleType")
            
    
    
    def _isWorking(self) -> bool:
        try:
            self._psutil = __import__('psutil')
        except ModuleNotFoundError:
            logging.warning("psutil can not be imported, therefore disk functionality not avaiable")
            return False
        return True
    
    def getReadCount(self) -> int:
        if isinstance(self._psutil, ModuleType):
            return self._psutil.disk_io_counters().read_count - self._initValues.read_count
        logging.error("Something went wrong, in disk plugin psutil is not ModuleType")
        return -1
    
    def getWriteCount(self) -> int:
        if isinstance(self._psutil, ModuleType):
            return self._psutil.disk_io_counters().write_count - self._initValues.write_count
        logging.error("Something went wrong, in disk plugin psutil is not ModuleType")
        return -1
    
    def getReadBytes(self) -> int:
        if isinstance(self._psutil, ModuleType):
            return self._psutil.disk_io_counters().read_bytes - self._initValues.read_bytes
        logging.error("Something went wrong, in disk plugin psutil is not ModuleType")
        return -1
    
    def getWriteBytes(self) -> int:
        if isinstance(self._psutil, ModuleType):
            return self._psutil.disk_io_counters().write_bytes - self._initValues.write_bytes
        logging.error("Something went wrong, in disk plugin psutil is not ModuleType")
        return -1
    
    def getReadTime(self) -> int:
        if isinstance(self._psutil, ModuleType):
            return self._psutil.disk_io_counters().read_time - self._initValues.read_time
        logging.error("Something went wrong, in disk plugin psutil is not ModuleType")
        return -1
    
    def getWriteTime(self) -> int:
        if isinstance(self._psutil, ModuleType):
            return self._psutil.disk_io_counters().write_time - self._initValues.write_time
        logging.error("Something went wrong, in disk plugin psutil is not ModuleType")
        return -1
    
    def getReadMergedCount(self) -> int:
        if isinstance(self._psutil, ModuleType):
            return self._psutil.disk_io_counters().read_merged_count - self._initValues.read_merged_count
        logging.error("Something went wrong, in disk plugin psutil is not ModuleType")
        return -1
    
    def getWriteMergedCount(self) -> int:
        if isinstance(self._psutil, ModuleType):
            return self._psutil.disk_io_counters().write_merged_count - self._initValues.write_merged_count
        logging.error("Something went wrong, in disk plugin psutil is not ModuleType")
        return -1
    
    def getBusyTime(self) -> int:
        if isinstance(self._psutil, ModuleType):
            return self._psutil.disk_io_counters().busy_time - self._initValues.busy_time
        logging.error("Something went wrong, in disk plugin psutil is not ModuleType")
        return -1
    
    def getStatusData(self,
                      dataType: str, 
                      duration: float = -1.0, 
                      overrideHumanReadableWithValue: Optional[bool] = None) -> List[Dict[str, Union[bool, int, float ,str]]]:
        logging.debug("get disk status data with type=%s, duration=%s, overrride humand readable=%s",
                     str(dataType),
                     str(duration),
                     str(overrideHumanReadableWithValue))
        if dataType == "write_count":
            return [self.convertNumber(self.getWriteCount(), "write count", "", overrideHumanReadableWithValue)]
        if dataType == "read_count":
            return [self.convertNumber(self.getReadCount(), "read count", "", overrideHumanReadableWithValue)]
        if dataType == "write_bytes":
            return [self.convertNumber(self.getWriteBytes(), "write", "B", overrideHumanReadableWithValue)]
        if dataType == "read_bytes":
            return [self.convertNumber(self.getReadBytes(), "read", "B", overrideHumanReadableWithValue)]
        if dataType == "write_time":
            return [self.convertNumber(self.getWriteTime()/1000, "write", "s", overrideHumanReadableWithValue)]
        if dataType == "read_time":
            return [self.convertNumber(self.getReadTime()/1000, "read", "s", overrideHumanReadableWithValue)]
        if dataType == "write_merged_count":
            return [self.convertNumber(self.getWriteMergedCount(), "merged write", "", overrideHumanReadableWithValue)]
        if dataType == "read_merged_count":
            return [self.convertNumber(self.getReadMergedCount(), "merged read", "", overrideHumanReadableWithValue)]
        if dataType == "busy_time":
            return [self.convertNumber(self.getBusyTime()/1000, "busy", "s", overrideHumanReadableWithValue)]
        return []
    
    
    def getConfigData(self,
                      dataType: str, 
                      overrideHumanReadableWithValue: Optional[bool] = None) -> List[Dict[str, Union[bool, int, float ,str]]]:
        logging.debug("get disk config data with type=%s, overrride humand readable=%s",
                     str(dataType),
                     str(overrideHumanReadableWithValue))
        return []
    
    
    def convertNumber(self, 
                      num: Union[int,float], 
                      name: str, 
                      unit: str, 
                      overrideHumanReadableWithValue: Optional[bool]
                ) -> Dict[str, Union[bool, int, float ,str]]:
        if overrideHumanReadableWithValue or (overrideHumanReadableWithValue is None and settings.DEFAULT_TO_HUMAN_READABLE_NUMBERS) :
            (value, unitPrefix) = convertNumbers.convertBinaryPrefix(num)
            return {name + "[" + unitPrefix + unit + "]" : value}
        
        return {name + "[" + unit + "]" : str(num)}
    
    
plugin = _diskPlugin()



import logging

from mpi4py import MPI

from typing import Any, List, Optional, Tuple


from JuMonC.tasks import taskSwitcher, taskPool
from JuMonC.models import dataStore



logger = logging.getLogger(__name__)



__comm = MPI.COMM_WORLD
__keep_running = True


    
def waitForCommands() -> None:
    data: Optional[List[int]] = None
    taskPool.setupTaskPool()
    while __keep_running:
        data = __comm.bcast(data, root=0)
        taskSwitcher.tasks.addTask(data)
            
def sendResults() -> None:    
    # recive needed dataID
    dataID: int = 0
    dataID = __comm.bcast(None, root=0)

    (rec_res_avai, result) = testResultAvaiable(dataID)

    if  rec_res_avai == 1:
        __comm.reduce(result, op = MPI.SUM, root = 0)
        dataStore.removeResult(dataID)

        
def testResultAvaiable(dataID: int) -> Tuple[int, Optional[Any]]:
    result_avaiable: int = 0
    result: Optional[Any] = None
    try:
        result = dataStore.getResult(dataID)
        result_avaiable = 1
    except KeyError:
        result_avaiable = 0
        
    rec_res_avai: int = 0
    __comm.Allreduce(result_avaiable, rec_res_avai, op=MPI.PROD)
    return (rec_res_avai, result)


from typing import Optional, Dict, Tuple


from flask import Request
from flask_login import UserMixin  # type: ignore


from JuMonC.handlers import base
from JuMonC.helpers.generateToken import generateToken
from JuMonC.authentication import scopes, scope_name
from JuMonC import settings


        
def parseUserToken(text: str) -> Tuple[str,int]:
    parts = text.split(":")
    return (parts[0], int(parts[1]))


tokens: Dict[str,int]= {}


if settings.USER_DEFINED_TOKEN is not None:
    user_tokens = settings.USER_DEFINED_TOKEN.split(";")
    for user_token in user_tokens:
        (token, scope) = parseUserToken(user_token)
        tokens[token] = scope


for scope in scopes.values():
    while True:
        token = generateToken()
        if token in tokens:
            pass
        break
    tokens[token] = scope
        



for (token, scope) in tokens.items():
    print(scope_name[scope] + ": " + token)

class Authenticated(UserMixin):
    pass



@base.login_manager.user_loader
def user_loader(tokenScope: str) -> Optional[Authenticated]:
    if tokenScope not in tokens:
        return None

    auth = Authenticated()
    auth.id = tokenScope
    return auth


def getTokenScope(token: Optional[str]) -> Optional[Authenticated]:
    if token not in tokens:
        return None
    
    auth = Authenticated()
    auth.id = token
    
    #auth.is_authenticated = True
    
    return auth


@base.login_manager.request_loader
def request_loader(request: Request) -> Optional[Authenticated]:
    token = request.args.get('token', default = None, type = str)
    return getTokenScope(token)


@base.login_manager.unauthorized_handler
def unauthorized_handler() -> str:
    return 'Unauthorized'


"""
JuMonC is a programm that lets you monitor and control you simulation using the provided REST-API.

Further informaton can be found in the README.md.
The application cmd arguments can be found in doc/CMD/Parameters.md
The REST API is documented with openapi, and rendered well by gitlab on:
https://gitlab.jsc.fz-juelich.de/witzler1/jumonc/-/blob/CI_fixes/doc/REST_API/openapi.yaml
"""


import logging
import sys


from JuMonC import settings

def setup_logging() -> None:
    handler = logging.StreamHandler(sys.stdout if settings.LOG_STDOUT else sys.stderr)
    formatter = logging.Formatter(settings.LOG_PREFIX + settings.LOG_FORMAT)
    handler.setFormatter(formatter)
    logging.getLogger().addHandler(handler)
    logging.getLogger().setLevel(settings.LOG_LEVEL)
    
    fileHandler = logging.FileHandler('JuMonC.log')
    fileHandler.setFormatter(formatter)
    logging.getLogger().addHandler(fileHandler)


setup_logging()


def reset_logging() -> None:
    logger = logging.getLogger()
    while logger.hasHandlers():
        logger.removeHandler(logger.handlers[0])
    setup_logging()


from JuMonC.tasks import LinuxNetwork


test_data = [{'interface': 'lo', 'received': {'Bytes': 30296808868, 'Packets': 9925597, 'Errs': 0, 'Drop': 0, 'Fifo': 0, 'Frame': 0, 'Compressed': 0, 'Multicast': 0}, 'transmitted': {'Bytes': 30296808868, 'Packets': 9925597, 'Errs': 0, 'Drop': 0, 'Fifo': 0, 'Frame': 0, 'Compressed': 0, 'Multicast': 0}}, {'interface': 'enp225s0f0np0', 'received': {'Bytes': 0, 'Packets': 0, 'Errs': 0, 'Drop': 0, 'Fifo': 0, 'Frame': 0, 'Compressed': 0, 'Multicast': 0}, 'transmitted': {'Bytes': 0, 'Packets': 0, 'Errs': 0, 'Drop': 0, 'Fifo': 0, 'Frame': 0, 'Compressed': 0, 'Multicast': 0}}, {'interface': 'enp225s0f1np1', 'received': {'Bytes': 0, 'Packets': 0, 'Errs': 0, 'Drop': 0, 'Fifo': 0, 'Frame': 0, 'Compressed': 0, 'Multicast': 0}, 'transmitted': {'Bytes': 0, 'Packets': 0, 'Errs': 0, 'Drop': 0, 'Fifo': 0, 'Frame': 0, 'Compressed': 0, 'Multicast': 0}}, {'interface': 'eno1', 'received': {'Bytes': 14480425514, 'Packets': 27025577, 'Errs': 6, 'Drop': 0, 'Fifo': 0, 'Frame': 6, 'Compressed': 0, 'Multicast': 77081}, 'transmitted': {'Bytes': 23424149724, 'Packets': 30228869, 'Errs': 0, 'Drop': 0, 'Fifo': 0, 'Frame': 0, 'Compressed': 0, 'Multicast': 0}}, {'interface': 'eno2', 'received': {'Bytes': 0, 'Packets': 0, 'Errs': 0, 'Drop': 0, 'Fifo': 0, 'Frame': 0, 'Compressed': 0, 'Multicast': 0}, 'transmitted': {'Bytes': 0, 'Packets': 0, 'Errs': 0, 'Drop': 0, 'Fifo': 0, 'Frame': 0, 'Compressed': 0, 'Multicast': 0}}, {'interface': 'enp33s0', 'received': {'Bytes': 4460048701958, 'Packets': 3462215440, 'Errs': 7007943, 'Drop': 1804099, 'Fifo': 0, 'Frame': 7007943, 'Compressed': 0, 'Multicast': 13458754}, 'transmitted': {'Bytes': 2682541949869, 'Packets': 2763418224, 'Errs': 0, 'Drop': 0, 'Fifo': 0, 'Frame': 0, 'Compressed': 0, 'Multicast': 0}}, {'interface': 'ib0', 'received': {'Bytes': 102736163162268, 'Packets': 27688284139, 'Errs': 0, 'Drop': 0, 'Fifo': 0, 'Frame': 0, 'Compressed': 0, 'Multicast': 0}, 'transmitted': {'Bytes': 98963885344636, 'Packets': 26079784462, 'Errs': 0, 'Drop': 0, 'Fifo': 0, 'Frame': 0, 'Compressed': 0, 'Multicast': 0}}, {'interface': 'ib0.8001', 'received': {'Bytes': 19203, 'Packets': 123, 'Errs': 0, 'Drop': 0, 'Fifo': 0, 'Frame': 0, 'Compressed': 0, 'Multicast': 0}, 'transmitted': {'Bytes': 1543, 'Packets': 15, 'Errs': 0, 'Drop': 0, 'Fifo': 0, 'Frame': 0, 'Compressed': 0, 'Multicast': 0}}, {'interface': 'ib0.8004', 'received': {'Bytes': 22673, 'Packets': 144, 'Errs': 0, 'Drop': 0, 'Fifo': 0, 'Frame': 0, 'Compressed': 0, 'Multicast': 0}, 'transmitted': {'Bytes': 2166, 'Packets': 21, 'Errs': 0, 'Drop': 0, 'Fifo': 0, 'Frame': 0, 'Compressed': 0, 'Multicast': 0}}, {'interface': 'ib0.8003', 'received': {'Bytes': 23019, 'Packets': 147, 'Errs': 0, 'Drop': 0, 'Fifo': 0, 'Frame': 0, 'Compressed': 0, 'Multicast': 0}, 'transmitted': {'Bytes': 2151, 'Packets': 21, 'Errs': 0, 'Drop': 0, 'Fifo': 0, 'Frame': 0, 'Compressed': 0, 'Multicast': 0}}, {'interface': 'ib0.8002', 'received': {'Bytes': 20958, 'Packets': 134, 'Errs': 0, 'Drop': 0, 'Fifo': 0, 'Frame': 0, 'Compressed': 0, 'Multicast': 0}, 'transmitted': {'Bytes': 2151, 'Packets': 21, 'Errs': 0, 'Drop': 0, 'Fifo': 0, 'Frame': 0, 'Compressed': 0, 'Multicast': 0}}]

def test_isWorking():
    plugin = LinuxNetwork.plugin
    assert(plugin.isWorking)
    

def test_Network_info():
    plugin = LinuxNetwork.plugin
    interfaces = plugin.GetNetworkInfo()
    assert(len(interfaces) > 0)


def test_InterfaceNames():
    plugin = LinuxNetwork.plugin
    a = plugin.getAvaiableInterfaces()
    b = plugin.getAvaiableInterfaces()
    assert(a)
    assert(a == b)

    
def test_networkDescriptions():
    plugin = LinuxNetwork.plugin
    assert(len(plugin.getAvaiableDataTypes()) == len(plugin.getAvaiableDataTypesDescriptions()))
        
def test_reduceInterfaces():
    LinuxNetwork.plugin.getAvaiableInterfaces()
    LinuxNetwork.plugin._interfaces.append("ib0")
    result = LinuxNetwork._reduceInterfaces(test_data, "ib0")
    assert(len(result) == 1)
    assert(result[0]["interface"] == "ib0")
    
def test_reduceDataTypes():
    LinuxNetwork.plugin.getAvaiableInterfaces()
    LinuxNetwork.plugin._interfaces.append("ib0")
    result = LinuxNetwork._reduceInterfaces(test_data, "ib0")
    result = LinuxNetwork._reduceDataTypes(result, "Packets")
    assert(len(result[0]) == 3)
    assert(result[0]["received"]["Packets"] == 27688284139)
    assert(result[0]["transmitted"]["Packets"] == 26079784462)
           
def test_getDataDiff():
    result = LinuxNetwork._getDataDiff(test_data, test_data)
    assert(result[0]["received"]["Packets"] == 0)


def test_divideData():
    LinuxNetwork.plugin.getAvaiableInterfaces()
    LinuxNetwork.plugin._interfaces.append("ib0")
    result = LinuxNetwork._divideData(test_data, 10.0, "/s")
    result = LinuxNetwork._reduceInterfaces(result, "ib0")
    result = LinuxNetwork._reduceDataTypes(result, "Packets/s")
    assert(result[0]["received"]["Packets/s"] >= 2768828413.8 and result[0]["received"]["Packets/s"] <= 27688284140.0)
    assert(result[0]["transmitted"]["Packets/s"] >= 2607978446.1 and result[0]["transmitted"]["Packets/s"] <= 2607978446.4)
    
def test_makeDataHumanReadable():
    LinuxNetwork.plugin.getAvaiableInterfaces()
    LinuxNetwork.plugin._interfaces.append("ib0")
    result = LinuxNetwork._reduceInterfaces(test_data, "ib0")
    result = LinuxNetwork._reduceDataTypes(result, "Packets")
    result = LinuxNetwork._makeDataHumanReadable(result)
    assert(len(result[0]) == 3)
    assert(result[0]["received"]["GiPackets"] == "25.79")
    assert(result[0]["transmitted"]["GiPackets"] == "24.29")
    
    
def test_getData():
    result = LinuxNetwork.plugin.getData("all", 0.1, "all", False)
    assert(len(result) > 0)
    assert(result[0]["interface"])
    assert(result[0]["received"])
    assert(isinstance(result[0]["received"]["Bytes/s"], float))
    assert(result[0]["transmitted"])
    
    result = LinuxNetwork.plugin.getData("all", -1.0, "all", False)
    assert(len(result) > 0)
    assert(result[0]["interface"])
    assert(result[0]["received"])
    assert(isinstance(result[0]["transmitted"]["Bytes"], int))
    assert(result[0]["transmitted"])


import logging


from typing import Callable, Any

from functools import wraps

from flask import make_response, Response, request
from flask_login import current_user  # type: ignore


from JuMonC import settings
from JuMonC.authentication import tokens


logger = logging.getLogger(__name__)


def check_auth(neededScope: int) -> Callable[[Callable[..., Response]], Callable[..., Response]]:
    def no_auth() -> Response:
        return make_response("You are not allowed to access this link, login using `/login` and your token. You can see the access level "
                             + "allowed with your token using `/scope` or supply a valid token using the token parameter", 401)
    
    def wrap(func: Callable[..., Response]) -> Callable[..., Response]:
        @wraps(func)
        def decorated_function(*args: Any, **kwargs: Any) -> Response:
            if settings.ENABLE_AUTH:
                # see if a token is supplied for this action
                token = request.args.get('token', default = None, type = str)
                if token is not None:
                    temp_user = tokens.getTokenScope(token)
                    if temp_user is not None and hasattr(temp_user,'id'):
                        if  tokens.tokens[temp_user.id] >= neededScope:
                            logging.debug("scope_temp_user: %s" , str(tokens.tokens[temp_user.id]))
                            logging.debug("scopeNeeded: %s", str(neededScope))
                            return func(*args, **kwargs)
                # see if the user is signed in
                if hasattr(current_user, 'id'):
                    if tokens.tokens[current_user.id] >= neededScope:
                        logging.debug("scope_logined_user: %s", str(tokens.tokens[current_user.id]))
                        logging.debug("scopeNeeded: %s", str(neededScope))
                        return func(*args, **kwargs)
                return no_auth()
            return func(*args, **kwargs)
            
        return decorated_function
    return wrap


import logging

from queue import Queue

from typing import List, Optional


from JuMonC.tasks import taskID
from JuMonC import settings



logger = logging.getLogger(__name__)



def notyetdefined(data: List[int]) -> None:
    print(data)

class taskSwitcher:
    pending_tasks: Queue = Queue(settings.PENDING_TASKS_SOFT_LIMIT + settings.MAX_THREADS_PER_TASK * settings.MAX_WORKER_THREADS + 1)
    
    def __init__(self) -> None:
        """Singelton class that switches taks depending on id."""
        self.switch_dic = {
            # General commands

            taskID.finalize : notyetdefined,


            # Job commands



            # CPU commands

            taskID.meassure_cpu_load : notyetdefined,
            taskID.retrieve_cpu_load : notyetdefined,

            taskID.meassure_cpu_frequency : notyetdefined,


            # GPU commands

            taskID.meassure_gpu_load : notyetdefined,
            taskID.retrieve_gpu_load : notyetdefined,

            taskID.meassure_gpu_frequency : notyetdefined,


            # Network commands



            # IO commands



            # General application commands



            # Alya commands
            
            
        }
        
    def executeNextTask(self) -> None:
        data = self.pending_tasks.get()
        logging.debug("Executing task: %s", str(data))
        task = self.switch_dic.get(data[0], lambda data: logging.warning("Invalid taskID: %s", str(data[0])))
        task(data)
        self.pending_tasks.task_done()
        
    def addTask(self, data: Optional[List[int]]) -> None:
        if data is not None:
            self.pending_tasks.put(data)

        
tasks = taskSwitcher()


from JuMonC.helpers import cmdArguments
from JuMonC._version import __REST_version__, REST_version_info, __version__
from JuMonC import settings


def test_Log_Format_setting():
    cmdArguments.parseCMDOptions(["--LOG_FORMAT", "[%(asctime)s]"])
    assert( settings.LOG_FORMAT == "[%(asctime)s]")
    
    # default:
    cmdArguments.parseCMDOptions([])
    assert( settings.LOG_FORMAT == "[%(asctime)s][PID:%(process)d][%(levelname)s][%(name)s] %(message)s")

    
def test_Log_Level_setting():
    cmdArguments.parseCMDOptions(["--LOG_LEVEL", "ERROR"])
    assert( settings.LOG_LEVEL == "ERROR")
    
    cmdArguments.parseCMDOptions(["--LOG_LEVEL", "WARN"])
    assert( settings.LOG_LEVEL == "WARN")
    
    cmdArguments.parseCMDOptions(["--LOG_LEVEL", "INFO"])
    assert( settings.LOG_LEVEL == "INFO")
    
    cmdArguments.parseCMDOptions(["--LOG_LEVEL", "DEBUG"])
    assert( settings.LOG_LEVEL == "DEBUG")
    
    # default:
    cmdArguments.parseCMDOptions([])
    assert( settings.LOG_LEVEL == "INFO")
    
    # different argument style
    cmdArguments.parseCMDOptions(["--LOG_LEVEL=DEBUG"])
    assert( settings.LOG_LEVEL == "DEBUG")

    
def test_Log_Stdout_setting():
    cmdArguments.parseCMDOptions(["--LOG_STDOUT"])
    assert( settings.LOG_STDOUT == True)
    
    # default:
    cmdArguments.parseCMDOptions([])
    assert( settings.LOG_STDOUT == False)


def test_Log_Prefix_setting():
    cmdArguments.parseCMDOptions(["--LOG_PREFIX", "[test]"])
    assert( settings.LOG_PREFIX == "[test]")
    
    # default:
    cmdArguments.parseCMDOptions([])
    assert( settings.LOG_PREFIX == "")


def test_MAX_WORKER_THREADS_setting():
    cmdArguments.parseCMDOptions(["--MAX_WORKER_THREADS", "35"])
    assert( settings.MAX_WORKER_THREADS == 35)
    
    # invalid value
    cmdArguments.parseCMDOptions(["--MAX_WORKER_THREADS", "-35"])
    assert( settings.MAX_WORKER_THREADS == 1)
    
    # default:
    cmdArguments.parseCMDOptions([])
    assert( settings.MAX_WORKER_THREADS == 4)

    
def test_ONLY_CHOOSEN_REST_API_VERSION_setting():
    cmdArguments.parseCMDOptions(["--ONLY_CHOOSEN_REST_API_VERSION"])
    assert( settings.ONLY_CHOOSEN_REST_API_VERSION == True)
    
    # default:
    cmdArguments.parseCMDOptions([])
    assert( settings.ONLY_CHOOSEN_REST_API_VERSION == False)


def test_PENDING_TASKS_SOFT_LIMIT_setting():
    cmdArguments.parseCMDOptions(["--PENDING_TASKS_SOFT_LIMIT", "35"])
    assert( settings.PENDING_TASKS_SOFT_LIMIT == 35)
    
    # invalid value
    cmdArguments.parseCMDOptions(["--PENDING_TASKS_SOFT_LIMIT", "-35"])
    assert( settings.PENDING_TASKS_SOFT_LIMIT == 1)
    
    # default:
    cmdArguments.parseCMDOptions([])
    assert( settings.PENDING_TASKS_SOFT_LIMIT == 100)


def test_REST_API_PORT_setting():
    cmdArguments.parseCMDOptions(["--REST_API_PORT", "3005"])
    assert( settings.REST_API_PORT == 3005)
    
    # shorthand
    cmdArguments.parseCMDOptions(["-p", "3500"])
    assert( settings.REST_API_PORT == 3500)
    
    # default:
    cmdArguments.parseCMDOptions([])
    assert( settings.REST_API_PORT == 12121)


def test_REST_API_VERSION_setting():
    cmdArguments.parseCMDOptions(["--REST_API_VERSION", "1"])
    assert( settings.REST_API_VERSION == 1)
    
    # default:
    cmdArguments.parseCMDOptions([])
    assert( settings.REST_API_VERSION == REST_version_info[0])


def test_SHORT_JOB_MAX_TIME_setting():
    cmdArguments.parseCMDOptions(["--SHORT_JOB_MAX_TIME", "1"])
    assert( settings.SHORT_JOB_MAX_TIME == 1.0)
    
    # default:
    cmdArguments.parseCMDOptions([])
    assert( settings.SHORT_JOB_MAX_TIME == 0.1)
    
        
def test_DEFAULT_TO_HUMAN_READABLE_NUMBERS_setting():
    cmdArguments.parseCMDOptions(["--DONT_DEFAULT_TO_HUMAN_READABLE_NUMBERS"])
    assert( settings.DEFAULT_TO_HUMAN_READABLE_NUMBERS == False)
    
    # default:
    cmdArguments.parseCMDOptions([])
    assert( settings.DEFAULT_TO_HUMAN_READABLE_NUMBERS == True)


import logging

from typing import Any, Dict, Optional

import threading



logger = logging.getLogger(__name__)


__data_lock: threading.Lock = threading.Lock()
__data: Dict[int, Optional[Any]] = {}

__id_lock: threading.Lock = threading.Lock()
__last_id: int = 0

def getNextDataID() -> int:
    global __last_id
    logging.debug("Waiting for next ID")
    with __id_lock:
        __last_id = __last_id + 1
        logging.debug("Next Id generated: %i", __last_id)
        return __last_id
    
def addResult(dataID: int, result: Optional[Any]) -> None:
    logging.debug("Waiting for access to data store")
    with __data_lock:
        __data[dataID] = result
        logging.debug("Data (%s) for ID %i stored", str(result), dataID)
    
def getResult(dataID: int) -> Optional[Any]:
    logging.debug("Waiting for access to data store")
    with __data_lock:
        result = __data[dataID]
        logging.debug("Data (%s) for ID %i retrived", str(result), dataID)
        return result
    
def removeResult(dataID: int) -> None:
    logging.debug("Waiting for access to data store")
    with __data_lock:
        del __data[dataID]
        logging.debug("Data for ID %i removed", dataID)


import logging

from flask import jsonify, make_response, Response

from typing import Dict, List, Union


from JuMonC.handlers.base import api_version_path, check_version, RESTAPI
from JuMonC.authentication import scopes
from JuMonC.authentication.check import check_auth
from JuMonC.handlers import cpu, gpu, io, job, main_memory, network


logger = logging.getLogger(__name__)

links: Dict[str, List[Dict[str, Union[bool, str, List[Dict[str, str]]]]]] = {}


@RESTAPI.route(api_version_path)
@check_version
@check_auth(scopes["see_links"])
def returnVersionLinks(version: int) -> Response:
    logging.debug("Accessed /v%i/", version)
    return make_response(jsonify(sorted(links["v" + str(version)], key=lambda dic: dic['link'])), 200)


def registerRestApiPaths(version: int) -> Dict[str, Union[bool, str, List[Dict[str, str]]]]:
    links_this_version: List[Dict[str, Union[bool, str, List[Dict[str, str]]]]] = []
    
    links_this_version.append(cpu.registerRestApiPaths(version))
    links_this_version.append(gpu.registerRestApiPaths(version))
    links_this_version.append(io.registerRestApiPaths(version))
    links_this_version.append(job.registerRestApiPaths(version))
    links_this_version.append(main_memory.registerRestApiPaths(version))
    links_this_version.append(network.registerRestApiPaths(version))
    
    links["v" + str(version)] = links_this_version
    
    
    
    return {
        "link": "/v" + str(version),
        "isOptional": False,
        "description": "Following this links leads to API version " + str(version),
        "parameters": [
             {"name": "token",
              "description": "Supply a token that shows you are allowed to access this link (or login once using \"/login\")"}
        ]
    }


"""The authentication package of JuMonC provides functionality to authenticate users using the REST-API."""


import logging


logger = logging.getLogger(__name__)

scope_name = {
    0: "see_links",
    5: "retrieve_data",
    10: "compute_data",
    15: "retrieve_simulation_data",
    20: "compute_simulation_data",
    100: "full"
}
    

scopes = {
    scope_name[0]: 0,
    scope_name[5]: 5,
    scope_name[10]: 10,
    scope_name[15]: 15,
    scope_name[20]: 20,
    scope_name[100]: 100
}


from JuMonC.helpers import startup
from JuMonC.models import pluginInformation

def test_findLocalPlugins():
    assert (pluginInformation.slurm_is_working == False)
    assert (pluginInformation.papi_is_working == False)
    assert (pluginInformation.nvidia_is_working == False)
    assert (pluginInformation.network_is_working == False)
    
    startup.findLocalPlugins()

    #assert (pluginInformation.slurm_is_working == True)
    #assert (pluginInformation.papi_is_working == True)
    #assert (pluginInformation.nvidia_is_working == True)
    assert (pluginInformation.network_is_working == True)


import logging

from flask import jsonify, make_response, Response

from typing import Dict, List, Union


from JuMonC.handlers.base import api_version_path, check_version, RESTAPI
from JuMonC.authentication import scopes
from JuMonC.authentication.check import check_auth


logger = logging.getLogger(__name__)

links: List[Dict[str, Union[str, Dict[str, str]]]] = []

gpu_path = "/gpu"

@RESTAPI.route(api_version_path + gpu_path, methods=["GET"])
@check_version
@check_auth(scopes["see_links"])
def returnGPULinks(version: int) -> Response:
    logging.debug("Accessed /v%i/gpu/", version)
    return make_response(jsonify(sorted(links, key=lambda dic: dic['link'])), 200)

def registerRestApiPaths(version: int) -> Dict[str, Union[bool, str, List[Dict[str, str]]]]:
    return {
        "link": "/v" + str(version) + gpu_path,
        "isOptional": False,
        "description": "Gather information concerning the GPU",
        "parameters": [
            {"name": "token",
             "description": "Supply a token that shows you are allowed to access this link (or login once using /login)"}]
    }


import logging

import mpi4py
from mpi4py import MPI

from threading import Thread

import sys


from JuMonC import settings
from JuMonC._version import __version__
from JuMonC.helpers.startup import communicateAvaiablePlugins
from JuMonC.helpers.cmdArguments import parseCMDOptions



logger = logging.getLogger(__name__)
logging.info("Running JuMonC with version: %s", __version__)

mpi4py.rc.threads = True
mpi4py.rc.thread_level = "funneled"

comm = MPI.COMM_WORLD
rank = comm.Get_rank()

def startJuMonC() -> None:
    parseCMDOptions(sys.argv[1:])
    communicateAvaiablePlugins()

    #pylint: disable=import-outside-toplevel
    if rank == 0:
        from JuMonC.handlers.base import RESTAPI
        from JuMonC.handlers import main
    
        from JuMonC.tasks import mpiroot
    
        main.registerRestApiPaths()
        flask_thread = Thread(target=RESTAPI.run, kwargs={'debug': True, 'port': settings.REST_API_PORT, 'use_reloader': False})
        flask_thread.start()
    
    
        mpiroot.waitForCommands()
    
    
        flask_thread.join()
    
    else:
        from JuMonC.tasks import mpihandler
    
        mpihandler.waitForCommands()
    #pylint: enable=import-outside-toplevel
        


import pytest

from JuMonC.models import dataStore


def test_ID():
    assert( (dataStore.getNextDataID() - dataStore.getNextDataID()) == -1)
    
def test_Store():
    test1 = True
    test2 = 3
    test3 = [3,5,7]
    test4 = 2.7
    test5 = "test"
    
    test1ID = dataStore.getNextDataID()
    test2ID = dataStore.getNextDataID()
    test3ID = dataStore.getNextDataID()
    test4ID = dataStore.getNextDataID()
    test5ID = dataStore.getNextDataID()
    
    dataStore.addResult(test1ID, test1)
    dataStore.addResult(test2ID, test2)
    
    assert( test1 == dataStore.getResult(test1ID))
    
    dataStore.addResult(test3ID, test3)
    
    assert( test1 == dataStore.getResult(test1ID))
    
    dataStore.removeResult(test1ID)
    
    with pytest.raises(Exception):
        assert dataStore.getResult(test1ID)
    
    dataStore.addResult(test4ID, test4)
    dataStore.addResult(test5ID, test5)
    
    assert( test2 == dataStore.getResult(test2ID))
    assert( test3 == dataStore.getResult(test3ID))
    assert( test4 == dataStore.getResult(test4ID))
    assert( test5 == dataStore.getResult(test5ID))


import time

from JuMonC.tasks import taskPool
from JuMonC import settings

def test_taskPool():
    settings.MAX_WORKER_THREADS = 4
    taskPool._task_pool_avaiable=False
    for i in range(8):
        simpleFunc()
    assert(taskPool._thread_pool._work_queue.qsize() == 4)
    time.sleep(0.5)
    assert(taskPool._thread_pool._work_queue.qsize() >= 0)
    time.sleep(0.5)
    assert(taskPool._thread_pool._work_queue.qsize() == 0)
    
@taskPool.executeAsTask
def simpleFunc():
    time.sleep(0.5)


from JuMonC.helpers import convertNumbers

def test_convertBinaryPrefix():
    (value, unit) = convertNumbers.convertBinaryPrefix(9999)
    assert(value == "9999")
    assert(unit == "")
    (value, unit) = convertNumbers.convertBinaryPrefix(9999.0)
    assert(value == "9999.00")
    assert(unit == "")
    
    (value, unit) = convertNumbers.convertBinaryPrefix(10240)
    assert(value == "10.00")
    assert(unit == "Ki")
    (value, unit) = convertNumbers.convertBinaryPrefix(10240.0)
    assert(value == "10.00")
    assert(unit == "Ki")
    
    (value, unit) = convertNumbers.convertBinaryPrefix(102400)
    assert(value == "100.00")
    assert(unit == "Ki")
    (value, unit) = convertNumbers.convertBinaryPrefix(102400.0)
    assert(value == "100.00")
    assert(unit == "Ki")


class Plugin:
    works = None
    
    def __init__(self) -> None:
        """Base class for all plugins to use."""
    
    def isWorking(self) -> bool:
        if self.works is None:
            self.works = self._isWorking()
            return self.works
        return self.works
    
    def _isWorking(self) -> bool:
        raise NotImplementedError("Please Implement this method")


# Module version stage suffix map
_specifier_ = {'alpha': 'a', 'beta': 'b', 'candidate': 'rc', 'final': ''}


# Module version
version_info = (0, 4, 0, 'candidate', 0)

# Module version accessible using JuMonC.__version__
version_end = '' if version_info[3] == 'final' else _specifier_[version_info[3]]+str(version_info[4])
__version__ = f'{version_info[0]}.{version_info[1]}.{version_info[2]}{version_end}' 


REST_version_info = (1, 0, 1, 'candidate', 0)

REST_version_end = '' if REST_version_info[3] == 'final' else _specifier_[REST_version_info[3]]+str(REST_version_info[4])
__REST_version__ = f'{REST_version_info[0]}.{REST_version_info[1]}.{REST_version_info[2]}{REST_version_end}' 



from JuMonC.tasks import memory


def test_isWorking():
    plugin = memory.plugin
    assert(plugin.isWorking)

def test_free():
    plugin = memory.plugin
    assert(plugin.getFree() > 0)

def test_used():
    plugin = memory.plugin
    assert(plugin.getUsed() > 0)

def test_total():
    plugin = memory.plugin
    assert(plugin.getTotal() > 0)

def test_getStatusData():
    plugin = memory.plugin
    assert(len(plugin.getStatusData("free")) == 1)
    assert(len(plugin.getStatusData("used")) == 1)
    assert(len(plugin.getStatusData("used", 1)) == 1)
    assert(len(plugin.getStatusData("used", 1, False)) == 1)
    assert(len(plugin.getStatusData("none_sense", 1)) == 0)

def test_getConfigData():
    plugin = memory.plugin
    assert(len(plugin.getConfigData("total")) == 1)
    assert(len(plugin.getConfigData("none_sense")) == 0)


from mpi4py import MPI
import logging

from JuMonC.models import pluginInformation


comm = MPI.COMM_WORLD
rank = comm.Get_rank()

logger = logging.getLogger(__name__)

def communicateAvaiablePlugins() -> None:
    findLocalPlugins()

def findLocalPlugins() -> None:
    test_papi()
    test_nv()
    test_linuxNet()
    test_cpu()
    test_mem()
    test_disk()
    
    
    # there are plugins like slurm that are only needed on the root node, therefore we only check for it there
    if rank == 0:
        test_slurm()


def test_papi() -> None:
    #pylint: disable=import-outside-toplevel
    from JuMonC.tasks.Papi import PapiPlugin
    #pylint: enable=import-outside-toplevel
        
    plugin_p = PapiPlugin()
    pluginInformation.papi_is_working = bool(plugin_p.isWorking())
    if pluginInformation.papi_is_working is False:
        if rank == 0:
            logging.info("Could not load PAPI plugin")
        else:
            logging.debug("Could not load PAPI plugin")
            

def test_nv() -> None:
    #pylint: disable=import-outside-toplevel
    from JuMonC.tasks.Nvidia import NvidiaPlugin
    #pylint: enable=import-outside-toplevel
        
    plugin_n = NvidiaPlugin()
    pluginInformation.nvidia_is_working = bool(plugin_n.isWorking())
    if pluginInformation.nvidia_is_working is False:
        if rank == 0:
            logging.info("Could not load Nvidia plugin")
        else:
            logging.debug("Could not load Nvidia plugin")
            
            
def test_linuxNet() -> None:
    #pylint: disable=import-outside-toplevel
    from JuMonC.tasks import LinuxNetwork
    #pylint: enable=import-outside-toplevel
        
    plugin_net = LinuxNetwork.plugin
    pluginInformation.network_is_working = bool(plugin_net.isWorking())
    if pluginInformation.network_is_working is False:
        if rank == 0:
            logging.info("Could not load Linux network plugin")
        else:
            logging.debug("Could not load Linux network plugin")


def test_cpu() -> None:
    #pylint: disable=import-outside-toplevel
    from JuMonC.tasks import CPU
    #pylint: enable=import-outside-toplevel
        
    plugin_cpu = CPU.plugin
    pluginInformation.cpu_is_working = bool(plugin_cpu.isWorking())
    if pluginInformation.cpu_is_working is False:
        if rank == 0:
            logging.info("Could not load Linux CPU plugin")
        else:
            logging.debug("Could not load Linux CPU plugin")
    
def test_mem() -> None:
    #pylint: disable=import-outside-toplevel
    from JuMonC.tasks import memory
    #pylint: enable=import-outside-toplevel
        
    plugin_mem = memory.plugin
    pluginInformation.memory_is_working = bool(plugin_mem.isWorking())
    if pluginInformation.memory_is_working is False:
        if rank == 0:
            logging.info("Could not load Linux memory plugin")
        else:
            logging.debug("Could not load Linux memory plugin")


def test_disk() -> None:
    #pylint: disable=import-outside-toplevel
    from JuMonC.tasks import disk
    #pylint: enable=import-outside-toplevel
        
    plugin_disk = disk.plugin
    pluginInformation.disk_is_working = bool(plugin_disk.isWorking())
    if pluginInformation.disk_is_working is False:
        if rank == 0:
            logging.info("Could not load Linux disk plugin")
        else:
            logging.debug("Could not load Linux disk plugin")
    

def test_slurm() -> None:
    #pylint: disable=import-outside-toplevel
    from JuMonC.tasks.Slurm import SlurmPlugin
    #pylint: enable=import-outside-toplevel
            
    plugin_s = SlurmPlugin()
    pluginInformation.slurm_is_working = bool(plugin_s.isWorking())
    if pluginInformation.slurm_is_working is False:
        logging.info("Could not load Slurm plugin")


import pathlib
from setuptools import setup

import JuMonC._version

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="JuMonC",
    version=JuMonC._version.__version__,
    description="Jülich Monitoring and Control",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://gitlab.jsc.fz-juelich.de/coec/jumonc",
    author="Christian Witzler",
    author_email="c.witzler@fz-juelich.de",
    license="BSD 3-Clause License",
    classifiers=[
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.9",
    ],
    packages=["JuMonC"],
    include_package_data=True,
    install_requires=["flask", "flask_login", "mpi4py"],
    entry_points={
        "console_scripts": [
            "JuMonC=JuMonC.start:startJuMonC",
        ]
    },
)



"""
The handlers package of JuMonC provides the handlers to interact with JuMonC using the REST-API.

The REST API is documented with openapi, and rendered well by gitlab on:
https://gitlab.jsc.fz-juelich.de/witzler1/jumonc/-/blob/CI_fixes/doc/REST_API/openapi.yaml
"""

import logging

from flask import jsonify, make_response, Response


from JuMonC.handlers.base import RESTAPI
from JuMonC._version import __version__
from JuMonC.tasks import taskID, taskSwitcher
from JuMonC.authentication import scopes
from JuMonC.authentication.check import check_auth


logger = logging.getLogger(__name__)


@RESTAPI.route("/ping", methods=["GET"])
def ping() -> str:
    return "PONG.\n"


@RESTAPI.route("/stop")
@check_auth(scopes["full"])
def stopJuMonC() -> Response:
    
    
    taskSwitcher.taskSwitcher().addTask([taskID.finalize])
    
    return make_response(jsonify("Stopping server and shutting down JuMonC\n"), 200)


@RESTAPI.route("/version", methods=["GET"])
def version() -> Response:
    response_body = {"JuMonC_version": __version__}
    return make_response(jsonify(response_body), 200)
